function [Population,Fitness,D_Dec,D_Pop,net,genFlag] = EnvironmentalSelectionOrig(Population,N,Problem,params,net,genFlag)
% The environmental selection of SPEA2 based on objective space

    [FrontNo,~] = NDSort(Population.objs,N);
    valid = FrontNo == 1;
    NDPop = Population(valid);
    temp1 = NDPop.decs;
    gen    = ceil(Problem.FE/Problem.N);
    maxgen = ceil(Problem.maxFE/Problem.N);
    if size(temp1,1) > 2&&isempty(find(isnan(temp1)==true))&& gen <= round(1*maxgen) && isempty(genFlag)
        [net,genFlag] = TrainGrowingGasNet(temp1,net,params,Problem,genFlag);
    end
    
    if Problem.D == 2
        PlotResults(net.w, net.C)
    elseif Problem.D == 3
        PlotResults1(net.w, net.C)
    end
    
     %% Calculate the fitness of each solution
    [D_Dec,D_Pop,Fitness] = CalFitness(Population.objs,Population.decs);
    
    %% Environmental selection
    Next = Fitness < 1;
    if sum(Next) < N
        [~,Rank] = sort(Fitness);
        Next(Rank(1:N)) = true;
    elseif sum(Next) > N
        Del  = Truncation(Population(Next).objs,Population(Next).decs,sum(Next)-N);
        Temp = find(Next);
        Next(Temp(Del)) = false;
    end
    % Population for next generation
    Population = Population(Next);
    Fitness    = Fitness(Next);
    D_Dec      = D_Dec(Next);
    D_Pop      = D_Pop(Next);
    % Sort the population
    [Fitness,rank] = sort(Fitness);
    Population = Population(rank);
    D_Dec = D_Dec(rank);
    D_Pop = D_Pop(rank);
end

function Del = Truncation(PopObj,PopDec,K)
% Select part of the solutions by truncation

    %% Truncation
    Distance_Pop = pdist2(PopObj,PopObj);
    Distance_Pop(logical(eye(length(Distance_Pop)))) = inf;
    
    Distance_Dec = pdist2(PopDec,PopDec);
    Distance_Dec(logical(eye(length(Distance_Dec)))) = inf;
    
    D = Distance_Pop + Distance_Dec;
    
    Del = false(1,size(PopObj,1));
    while sum(Del) < K
        Remain   = find(~Del);
        Temp     = sort(D(Remain,Remain),2);
        [~,Rank] = sortrows(Temp);
        Del(Remain(Rank(1))) = true;
    end
end